@@ -13,14 +13,7 @@ class Test_DB_Storage_Ibase extends Test_DB_Storage_Test
   
   public function testInit()
   {
-    $params = array("package"  => "sabel.db.ibase",
-                    "host"     => "localhost",
-                    "user"     => "develop",
-                    "password" => "develop",
-                    "database" => "/home/firebird/sdb_test.fdb");
-    
-    Sabel_DB_Config::add("default", $params);
-    
+    Sabel_DB_Config::add("default", Test_DB_TestConfig::getIbaseConfig());
     MODEL("SblStorage")->delete();
   }
 }
