@@ -188,12 +188,12 @@ class Test_DB_Validate extends SabelTestCase
     } elseif (extension_loaded("pgsql")) {
       $params = array("package"  => "sabel.db.pgsql",
                       "host"     => "127.0.0.1",
-                      "user"     => "pgsql",
-                      "password" => "pgsql",
+                      "user"     => "root",
+                      "password" => "",
                       "database" => "sdb_test");
     } elseif (extension_loaded("pdo_sqlite")) {
       $params = array("package"  => "sabel.db.pdo.sqlite",
-                      "database" => SABEL_BASE . "/Test/data/sdb_test.sq3");
+                      "database" => "/usr/local/lib/php/Sabel/Test/data/sdb_test.sq3");
     } else {
       return false;
     }
