@@ -10,12 +10,12 @@ class TestProcessor_Response extends Sabel_Bus_Processor
     
     $response->setResponses($responses);
     
-    if ($response->isServerError()) {
+    if ($response->getStatus()->isServerError()) {
       $exception = Sabel_Context::getContext()->getException();
       if (!is_object($exception)) return;
       
-      $eol = (ENVIRONMENT === DEVELOPMENT) ? "<br/>" : PHP_EOL;
-      $msg = "Exception: (" . get_class($exception) . ") "
+      $eol = (ENVIRONMENT === DEVELOPMENT) ? "<br />" : PHP_EOL;
+      $msg = get_class($exception) . ": "
            . $exception->getMessage()  . $eol
            . "At: " . date("r") . $eol . $eol
            . Sabel_Exception_Printer::printTrace($exception, $eol, true);
@@ -26,7 +26,7 @@ class TestProcessor_Response extends Sabel_Bus_Processor
         $response->setResponse("exception_message", $msg);
       }
       
-      l(preg_replace('/<br ?\/?>/', PHP_EOL, $msg), SBL_LOG_ERR);
+      l(PHP_EOL . str_replace("<br />", PHP_EOL, $msg), SBL_LOG_ERR);
     }
   }
   
