@@ -20,6 +20,111 @@ function unshift_include_paths($paths, $prefix = "")
   set_include_path($path . get_include_path());
 }
 
+function htmlescape($str, $charset = null)
+{
+  if ($charset !== null) {
+    return htmlentities($str, ENT_QUOTES, $charset);
+  }
+  
+  static $internalEncoding = null;
+  
+  if ($internalEncoding === null) {
+    if (extension_loaded("mbstring")) {
+      $internalEncoding = ini_get("mbstring.internal_encoding");
+      if ($internalEncoding === "") $internalEncoding = false;
+    } else {
+      $internalEncoding = false;
+    }
+  }
+  
+  if ($internalEncoding) {
+    return htmlentities($str, ENT_QUOTES, $internalEncoding);
+  } else {
+    return htmlentities($str, ENT_QUOTES);
+  }
+}
+
+function remove_nullbyte($arg)
+{
+  if (is_string($arg)) {
+    return str_replace("\000", "", $arg);
+  } elseif (is_array($arg)) {
+    foreach ($arg as &$v) {
+      if (is_string($v)) {
+        $v = str_replace("\000", "", $v);
+      }
+    }
+    
+    return $arg;
+  } else {
+    return $arg;
+  }
+}
+
+function get_temp_dir()
+{
+  static $exists = null;
+  
+  if ($exists === null) {
+    $exists = function_exists("sys_get_temp_dir");
+  }
+  
+  if ($exists) {
+    return sys_get_temp_dir();
+  } elseif (isset($_ENV["TMP"])) {
+    return realpath($_ENV["TMP"]);
+  } elseif (isset($_ENV["TMPDIR"])) {
+    return realpath($_ENV["TMPDIR"]);
+  } elseif (isset($_ENV["TEMP"])) {
+    return realpath($_ENV["TEMP"]);
+  } else {
+    if ($tmpFile = tempnam(md5hash(), "sbl_")) {
+      $dirName = realpath(dirname($tmpFile));
+      unlink($tmpFile);
+      return $dirName;
+    } else {
+      return null;
+    }
+  }
+}
+
+if (!function_exists("lcfirst")) {
+  function lcfirst($str)
+  {
+    if (!is_string($str) || $str === "") {
+      return "";
+    } else {
+      $str{0} = strtolower($str{0});
+      return $str;
+    }
+  }
+}
+
+function now()
+{
+  return date("Y-m-d H:i:s");
+}
+
+function md5hash()
+{
+  return md5(uniqid(mt_rand(), true));
+}
+
+function load($class, $config = null)
+{
+  static $container = null;
+  
+  if ($container === null) {
+    $container = Sabel_Container::create();
+  }
+  
+  if ($config === null) {
+    return $container->load($class);
+  } else {
+    return $container->load($class, $config);
+  }
+}
+
 function l($message, $level = SBL_LOG_INFO, $identifier = "default")
 {
   Sabel_Logger::create()->write($message, $level, $identifier);
@@ -28,12 +133,10 @@ function l($message, $level = SBL_LOG_INFO, $identifier = "default")
 function uri($uriParameter, $secure = false, $absolute = false)
 {
   $context = Sabel_Context::getContext();
+  $uriPrefix = "";
   
   if ($secure || $absolute) {
-    $protocol  = ($secure) ? "https" : "http";
-    $uriPrefix = $protocol . "://" . $context->getBus()->get("request")->getHttpHeader("host");
-  } else {
-    $uriPrefix = "";
+    $uriPrefix = (($secure) ? "https" : "http") . "://" . $_SERVER["SERVER_NAME"];
   }
   
   if (defined("URI_IGNORE")) {
@@ -55,16 +158,6 @@ function realempty($value)
   return ($value === null || $value === array() || $value === "");
 }
 
-function lcfirst($string)
-{
-  if (realempty($string) || !is_string($string)) {
-    return "";
-  } else {
-    $string{0} = strtolower($string{0});
-    return $string;
-  }
-}
-
 function dump()
 {
   if (PHP_SAPI === "cli") {
@@ -86,9 +179,9 @@ function dump()
   }
 }
 
-function environment($string)
+function environment($str)
 {
-  switch (strtolower($string)) {
+  switch (strtolower($str)) {
     case "production":
       return PRODUCTION;
     case "test":
@@ -100,35 +193,6 @@ function environment($string)
   }
 }
 
-function now()
-{
-  return date("Y-m-d H:i:s");
-}
-
-function htmlescape($string, $charset = null)
-{
-  if ($charset !== null) {
-    return htmlentities($string, ENT_QUOTES, $charset);
-  }
-  
-  static $internalEncoding = null;
-  
-  if ($internalEncoding === null) {
-    if (extension_loaded("mbstring")) {
-      $internalEncoding = ini_get("mbstring.internal_encoding");
-      if ($internalEncoding === "") $internalEncoding = false;
-    } else {
-      $internalEncoding = false;
-    }
-  }
-  
-  if ($internalEncoding) {
-    return htmlentities($string, ENT_QUOTES, $internalEncoding);
-  } else {
-    return htmlentities($string, ENT_QUOTES);
-  }
-}
-
 /***   sabel.db functions   ***/
 
 function is_model($model)
