@@ -19,7 +19,7 @@ class Sabel_Cache_Memcache implements Sabel_Cache_Interface
   {
     if (extension_loaded("memcache")) {
       $this->memcache = new Memcache();
-      $this->memcache->connect($server, $port);
+      $this->addServer($server, $port);
     } else {
       throw new Sabel_Exception_Runtime("memcache extension not loaded.");
     }
@@ -34,6 +34,11 @@ class Sabel_Cache_Memcache implements Sabel_Cache_Interface
     return self::$instance;
   }
   
+  public function addServer($server, $port = 11211, $weight = 1)
+  {
+    $this->memcache->addServer($server, $port, true, $weight);
+  }
+  
   public function read($key)
   {
     $result = $this->memcache->get($key);
