@@ -11,18 +11,37 @@
  */
 class Sabel_DB_Join extends Sabel_Object
 {
-  private
-    $model    = null,
-    $objects  = array(),
-    $joinType = "INNER",
-    $tblName  = "";
-    
+  /**
+   * @var string
+   */
+  protected $joinType = "INNER";
+  
+  /**
+   * @var Sabel_DB_Model
+   */
+  protected $model = null;
+  
+  /**
+   * @var object[]
+   */
+  protected $objects = array();
+  
+  /**
+   * @var array
+   */
+  protected $projection = array();
+  
+  /**
+   * @var string
+   */
+  protected $tblName = "";
+  
   public function __construct($model)
   {
     if (is_string($model)) {
       $model = MODEL($model);
     } elseif (!is_model($model)) {
-      $message = "argument must be a string or instance of model.";
+      $message = __METHOD__ . "() argument must be a string or an instance of model.";
       throw new Sabel_Exception_InvalidArgument($message);
     }
     
@@ -41,6 +60,20 @@ class Sabel_DB_Join extends Sabel_Object
     $this->joinType = $joinType;
   }
   
+  public function clear()
+  {
+    if (is_object($this->structure)) {
+      $this->structure->clear();
+    }
+    
+    Sabel_DB_Join_ColumnHash::clear();
+  }
+  
+  public function setProjection(array $projections)
+  {
+    $this->projection = $projections;
+  }
+  
   public function setCondition($arg1, $arg2 = null)
   {
     $this->model->setCondition($arg1, $arg2);
@@ -51,6 +84,16 @@ class Sabel_DB_Join extends Sabel_Object
     $this->model->setOrderBy($orderBy);
   }
   
+  public function setLimit($limit)
+  {
+    $this->model->setLimit($limit);
+  }
+  
+  public function setOffset($offset)
+  {
+    $this->model->setOffset($offset);
+  }
+  
   public function add($object, $alias = "", $joinKey = array())
   {
     if (is_string($object)) {
@@ -60,14 +103,13 @@ class Sabel_DB_Join extends Sabel_Object
     }
     
     $object->setChildName($this->tblName);
+    $this->structure->addJoinObject($this->tblName, $object);
     $this->objects[] = $object;
-    $this->structure->addJoinObject($object);
-    $this->structure->add($this->tblName, $object->getName());
     
-    if (!empty($joinKey)) return $this;
-    
-    $name = $object->getModel()->getTableName();
-    $object->setJoinKey(create_join_key($this->model, $name));
+    if (empty($joinKey)) {
+      $name = $object->getModel()->getTableName();
+      $object->setJoinKey(create_join_key($this->model, $name));
+    }
     
     return $this;
   }
@@ -99,6 +141,12 @@ class Sabel_DB_Join extends Sabel_Object
     return (int)$rows[0]["cnt"];
   }
   
+  public function selectOne($joinType = null)
+  {
+    $results = $this->select($joinType);
+    return (isset($results[0])) ? $results[0] : null;
+  }
+  
   public function select($joinType = null)
   {
     if ($joinType === null) {
@@ -106,31 +154,16 @@ class Sabel_DB_Join extends Sabel_Object
     }
     
     $stmt = $this->model->prepareStatement(Sabel_DB_Statement::SELECT);
-    $projection = $this->model->getProjection();
-    
-    if (empty($projection)) {
-      $projection = array();
-      foreach ($this->objects as $object) {
-        $projection = array_merge($projection, $object->getProjection($stmt));
-      }
-      
-      $quotedTblName = $stmt->quoteIdentifier($this->tblName);
-      foreach ($this->model->getColumnNames() as $column) {
-        $projection[] = $quotedTblName . "." . $stmt->quoteIdentifier($column);
-      }
-      
-      $projection = implode(", ", $projection);
-    }
+    $projection = $this->createProjection($stmt);
     
     $query = array();
     foreach ($this->objects as $object) {
       $query[] = $object->getJoinQuery($stmt, $joinType);
     }
     
+    $results = array();
     if ($rows = $this->execute($stmt, $projection, implode("", $query))) {
       $results = Sabel_DB_Join_Result::build($this->model, $this->structure, $rows);
-    } else {
-      $results = false;
     }
     
     $this->clear();
@@ -147,9 +180,36 @@ class Sabel_DB_Join extends Sabel_Object
     return $stmt->constraints($constraints)->execute();
   }
   
-  public function clear()
+  protected function createProjection(Sabel_DB_Statement $stmt)
   {
-    $this->structure->clear();
-    Sabel_DB_Join_ColumnHash::clear();
+    if (empty($this->projection)) {
+      $projection = array();
+      foreach ($this->objects as $object) {
+        $projection = array_merge($projection, $object->getProjection($stmt));
+      }
+      
+      $quotedTblName = $stmt->quoteIdentifier($this->tblName);
+      foreach ($this->model->getColumnNames() as $column) {
+        $projection[] = $quotedTblName . "." . $stmt->quoteIdentifier($column);
+      }
+    } else {
+      $projection = array();
+      foreach ($this->projection as $name => $proj) {
+        if (($tblName = convert_to_tablename($name)) === $this->tblName) {
+          foreach ($proj as $column) {
+            $projection[] = $stmt->quoteIdentifier($tblName) . "." . $stmt->quoteIdentifier($column);
+          }
+        } else {
+          foreach ($proj as $column) {
+            $as = "{$tblName}.{$column}";
+            if (strlen($as) > 30) $as = Sabel_DB_Join_ColumnHash::toHash($as);
+            $p = $stmt->quoteIdentifier($tblName) . "." . $stmt->quoteIdentifier($column);
+            $projection[] = $p . " AS " . $stmt->quoteIdentifier($as);
+          }
+        }
+      }
+    }
+    
+    return implode(", ", $projection);
   }
 }
