@@ -17,16 +17,6 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   protected $connectionName = "default";
   
-  /**
-   * @var string
-   */
-  protected $method = "";
-  
-  /**
-   * @var array
-   */
-  protected $arguments = array();
-  
   /**
    * @var string
    */
@@ -77,6 +67,11 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   protected $autoReinit = true;
   
+  /**
+   * @var array
+   */
+  protected $primaryKeyValues = array();
+  
   /**
    * @var array
    */
@@ -87,6 +82,11 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   protected $updateValues = array();
   
+  /**
+   * @var string
+   */
+  protected $versionColumn = "";
+  
   public function __construct($id = null)
   {
     $this->initialize();
@@ -173,6 +173,18 @@ abstract class Sabel_DB_Model extends Sabel_Object
     return $this;
   }
   
+  /**
+   * @param array $values
+   *
+   * @return self
+   */
+  public function setUpdateValues(array $values)
+  {
+    $this->updateValues = $values;
+    
+    return $this;
+  }
+  
   /**
    * @param string $key
    *
@@ -226,14 +238,6 @@ abstract class Sabel_DB_Model extends Sabel_Object
     return $this->modelName;
   }
   
-  /**
-   * @return array
-   */
-  public function getColumnNames()
-  {
-    return array_keys($this->metaCols);
-  }
-  
   /**
    * @return Sabel_DB_Metadata_Table
    */
@@ -250,6 +254,29 @@ abstract class Sabel_DB_Model extends Sabel_Object
     return $this->metaCols;
   }
   
+  /**
+   * @return array
+   */
+  public function getColumnNames()
+  {
+    return array_keys($this->metaCols);
+  }
+  
+  /**
+   * @return string
+   */
+  public function getVersionColumn()
+  {
+    $name = null;
+    if ($this->versionColumn !== "") {
+      $name = $this->versionColumn;
+    } elseif (isset($this->metaCols["version"])) {
+      $name = "version";
+    }
+    
+    return $name;
+  }
+  
   /**
    * @return array
    */
@@ -281,75 +308,29 @@ abstract class Sabel_DB_Model extends Sabel_Object
   public function setProperties(array $properties)
   {
     $pkey = $this->metadata->getPrimaryKey();
+    $this->values = $properties;
+    
+    if (empty($pkey)) return;
+    
+    $this->selected = true;
     if (is_string($pkey)) $pkey = (array)$pkey;
-    $selected = false;
     
-    if (!empty($pkey)) {
-      foreach ($pkey as $key) {
-        if (!isset($properties[$key])) {
-          $selected = false;
-          break;
-        }
+    foreach ($pkey as $key) {
+      if (!isset($properties[$key])) {
+        $this->selected = false;
+        break;
       }
-      
-      $selected = true;
     }
     
-    $this->values   = $properties;
-    $this->selected = $selected;
-    
-    return $this;
-  }
-  
-  /**
-   * @param string $method
-   * @param array  $args
-   *
-   * @return self
-   */
-  protected function prepare($method, array $args = array())
-  {
-    if (is_string($method)) {
-      $this->arguments = $args;
-      $this->method = $method;
-    } else {
-      $message = "method name must be string.";
-      throw new Sabel_Exception_InvalidArgument($message);
+    if ($this->selected) {
+      foreach ($pkey as $key) {
+        $this->primaryKeyValues[$key] = $this->$key;
+      }
     }
     
     return $this;
   }
   
-  /**
-   * @return mixed
-   */
-  protected function execute()
-  {
-    $result = null;
-    $method = $this->method;
-    
-    $beforeMethod = "before" . ucfirst($method);
-    $afterMethod  = "after"  . ucfirst($method);
-    
-    if ($this->hasMethod($beforeMethod)) {
-      $result = $this->$beforeMethod();
-    }
-    
-    if ($result === null) {
-      $execMethod = "_" . $method;
-      $result = $this->$execMethod();
-    }
-    
-    if ($this->hasMethod($afterMethod)) {
-      $afterResult = $this->$afterMethod($result);
-      if ($afterResult !== null) $result = $afterResult;
-    }
-    
-    if ($this->autoReinit) $this->initState();
-    
-    return $result;
-  }
-  
   /**
    * @param boolean $bool
    *
@@ -366,10 +347,7 @@ abstract class Sabel_DB_Model extends Sabel_Object
   public function initState()
   {
     $this->clearCondition(true);
-    
-    $this->method     = "";
     $this->projection = array();
-    $this->arguments  = array();
   }
   
   /**
@@ -440,11 +418,12 @@ abstract class Sabel_DB_Model extends Sabel_Object
   {
     if (is_string($orderBy)) {
       $this->constraints["order"] = $orderBy;
-      return $this;
     } else {
-      $message = "argument must be a string.";
+      $message = __METHOD__ . "() argument must be a string.";
       throw new Sabel_Exception_InvalidArgument($message);
     }
+    
+    return $this;
   }
   
   /**
@@ -457,11 +436,12 @@ abstract class Sabel_DB_Model extends Sabel_Object
   {
     if (is_numeric($limit)) {
       $this->constraints["limit"] = $limit;
-      return $this;
     } else {
-      $message = "argument must be an integer.";
+      $message = __METHOD__ . "() argument must be an integer.";
       throw new Sabel_Exception_InvalidArgument($message);
     }
+    
+    return $this;
   }
   
   /**
@@ -474,11 +454,12 @@ abstract class Sabel_DB_Model extends Sabel_Object
   {
     if (is_numeric($offset)) {
       $this->constraints["offset"] = $offset;
-      return $this;
     } else {
-      $message = "argument must be an integer.";
+      $message = __METHOD__ . "() argument must be an integer.";
       throw new Sabel_Exception_InvalidArgument($message);
     }
+    
+    return $this;
   }
   
   /**
@@ -519,26 +500,31 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   public function getCount($arg1 = null, $arg2 = null)
   {
-    return $this->prepare("getCount", array($arg1, $arg2))->execute();
-  }
-  
-  /**
-   * @return int
-   */
-  protected function _getCount()
-  {
-    @list ($arg1, $arg2) = $this->arguments;
-    $this->setCondition($arg1, $arg2);
+    $result = null;
+    if ($this->hasMethod("beforeSelect")) {
+      $result = $this->beforeSelect("getCount");
+    }
     
-    $projection = $this->projection;
-    $this->projection = "COUNT(*) AS cnt";
+    if ($result === null) {
+      $this->setCondition($arg1, $arg2);
+      $projection = $this->projection;
+      $this->projection = "COUNT(*) AS cnt";
+      
+      $stmt = $this->prepareStatement(Sabel_DB_Statement::SELECT);
+      $rows = $this->prepareSelect($stmt)->execute();
+      
+      $this->projection = $projection;
+      $result = (int)$rows[0]["cnt"];
+    }
     
-    $stmt = $this->prepareStatement(Sabel_DB_Statement::SELECT);
-    $rows = $this->prepareSelect($stmt)->execute();
+    if ($this->hasMethod("afterSelect")) {
+      $afterResult = $this->afterSelect($result, "getCount");
+      if ($afterResult !== null) $result = $afterResult;
+    }
     
-    $this->projection = $projection;
+    if ($this->autoReinit) $this->initState();
     
-    return (int)$rows[0]["cnt"];
+    return $result;
   }
   
   /**
@@ -549,38 +535,31 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   public function selectOne($arg1 = null, $arg2 = null)
   {
-    return $this->prepare("selectOne", array($arg1, $arg2))->execute();
-  }
-  
-  /**
-   * @throws Sabel_DB_Exception
-   * @return Sabel_DB_Model
-   */
-  protected function _selectOne()
-  {
-    @list ($arg1, $arg2) = $this->arguments;
-    
-    if ($arg1 === null && $this->condition === null) {
-      throw new Sabel_DB_Exception("selectOne() must set the condition.");
+    $result = null;
+    if ($this->hasMethod("beforeSelect")) {
+      $result = $this->beforeSelect("selectOne");
     }
     
-    $this->setCondition($arg1, $arg2);
-    $model = MODEL($this->modelName);
-    $this->_doSelectOne($model);
+    if ($result === null) {
+      if ($arg1 === null && $this->condition === null) {
+        $message = __METHOD__ . "() must set the condition.";
+        throw new Sabel_DB_Exception($message);
+      }
+      
+      $this->setCondition($arg1, $arg2);
+      $model = MODEL($this->modelName);
+      $this->_doSelectOne($model);
+      $result = $model;
+    }
     
-    return $model;
-  }
-  
-  /**
-   * @param Sabel_DB_Model $model
-   *
-   * @return void
-   */
-  protected function _doSelectOne(Sabel_DB_Model $model)
-  {
-    $stmt = $this->prepareStatement(Sabel_DB_Statement::SELECT);
-    $rows = $this->prepareSelect($stmt)->execute();
-    if (isset($rows[0])) $model->setProperties($rows[0]);
+    if ($this->hasMethod("afterSelect")) {
+      $afterResult = $this->afterSelect($result, "selectOne");
+      if ($afterResult !== null) $result = $afterResult;
+    }
+    
+    if ($this->autoReinit) $this->initState();
+    
+    return $result;
   }
   
   /**
@@ -591,98 +570,26 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   public function select($arg1 = null, $arg2 = null)
   {
-    return $this->prepare("select", array($arg1, $arg2))->execute();
-  }
-  
-  /**
-   * @return Sabel_DB_Model[]
-   */
-  protected function _select()
-  {
-    @list ($arg1, $arg2) = $this->arguments;
-    
-    $this->setCondition($arg1, $arg2);
-    $stmt = $this->prepareStatement(Sabel_DB_Statement::SELECT);
-    $rows = $this->prepareSelect($stmt)->execute();
-    
-    return (empty($rows)) ? array() : $this->toModels($rows);
-  }
-  
-  /**
-   * @param mixed $arg1
-   * @param mixed $arg2
-   *
-   * @return Sabel_DB_Model[]
-   */
-  public function selectWithChildren($child, $orderBy = "")
-  {
-    return $this->prepare("selectWithChildren", array($child, $orderBy))->execute();
-  }
-  
-  /**
-   * @return Sabel_DB_Model[]
-   */
-  protected function _selectWithChildren()
-  {
-    @list ($child, $orderBy) = $this->arguments;
-    $this->arguments = array();
-    
-    $ids    = array();
-    $pkey   = $this->metadata->getPrimaryKey();
-    $models = $this->_select();
-    
-    if (empty($models)) return array();
-    
-    foreach ($models as $model) {
-      $ids[] = $model->$pkey;
+    $result = null;
+    if ($this->hasMethod("beforeSelect")) {
+      $result = $this->beforeSelect("select");
     }
     
-    $childModel = (is_string($child)) ? MODEL($child) : $child;
-    if ($orderBy !== "") $childModel->setOrderBy($orderBy);
-    
-    $joinkey = create_join_key($childModel, $this->tableName);
-    $fkey = $joinkey["fkey"];
-    
-    $in = Sabel_DB_Condition::create(Sabel_DB_Condition::IN, $fkey, $ids);
-    $childModel->setCondition($in);
-    $results = $childModel->_select();
-    
-    $children = array();
-    foreach ($results as $result) {
-      $children[$result->$fkey][] = $result;
+    if ($result === null) {
+      $this->setCondition($arg1, $arg2);
+      $stmt = $this->prepareStatement(Sabel_DB_Statement::SELECT);
+      $rows = $this->prepareSelect($stmt)->execute();
+      $result = (empty($rows)) ? array() : $this->toModels($rows);
     }
     
-    unset($results);
-    
-    $childName = $childModel->getName();
-    foreach ($models as $model) {
-      if (isset($children[$model->$pkey])) {
-        $model->$childName = $children[$model->$pkey];
-      } else {
-        $model->$childName = array();
-      }
+    if ($this->hasMethod("afterSelect")) {
+      $afterResult = $this->afterSelect($result, "select");
+      if ($afterResult !== null) $result = $afterResult;
     }
     
-    return $models;
-  }
-  
-  /**
-   * @param array $rows
-   *
-   * @return Sabel_DB_Model[]
-   */
-  protected function toModels(array $rows)
-  {
-    $results = array();
-    $source  = MODEL($this->modelName);
+    if ($this->autoReinit) $this->initState();
     
-    foreach ($rows as $row) {
-      $model = clone $source;
-      $model->setProperties($row);
-      $results[] = $model;
-    }
-    
-    return $results;
+    return $result;
   }
   
   /**
@@ -692,74 +599,125 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   public function save(array $additionalValues = array())
   {
-    return $this->prepare("save", array($additionalValues))->execute();
-  }
-  
-  /**
-   * @return Sabel_DB_Model
-   */
-  protected function _save()
-  {
-    @list ($additionalValues) = $this->arguments;
-    
     if ($this->isSelected()) {
       $this->updateValues = array_merge($this->updateValues, $additionalValues);
-      return $this->_saveUpdate();
+      $result = $this->saveUpdate();
     } else {
       $this->values = array_merge($this->values, $additionalValues);
-      $this->_saveInsert();
+      $result = $this->saveInsert();
     }
+    
+    if ($this->autoReinit) $this->initState();
+    
+    return $result;
   }
   
   /**
    * @return array
    */
-  protected function _saveInsert()
+  protected function saveInsert()
   {
-    $columns = $this->getColumns();
-    
-    $saveValues = array();
-    foreach ($this->values as $k => $v) {
-      $saveValues[$k] = (isset($columns[$k])) ? $columns[$k]->cast($v) : $v;
+    $result = null;
+    if ($this->hasMethod("beforeInsert")) {
+      $result = $this->beforeInsert("save");
     }
     
-    $stmt  = $this->prepareStatement(Sabel_DB_Statement::INSERT);
-    $newId = $this->prepareInsert($stmt, $saveValues)->execute();
-    
-    foreach ($columns as $name => $column) {
-      if (!array_key_exists($name, $saveValues)) {
-        $this->$name = $column->default;
+    if ($result === null) {
+      $columns = $this->getColumns();
+      $saveValues = array();
+      
+      foreach ($this->values as $k => $v) {
+        $saveValues[$k] = (isset($columns[$k])) ? $columns[$k]->cast($v) : $v;
       }
+      
+      $stmt  = $this->prepareStatement(Sabel_DB_Statement::INSERT);
+      $newId = $this->prepareInsert($stmt, $saveValues)->execute();
+      
+      foreach ($columns as $name => $column) {
+        if (!array_key_exists($name, $saveValues)) {
+          $this->$name = $column->default;
+        }
+      }
+      
+      if ($newId !== null && ($field = $this->metadata->getSequenceColumn()) !== null) {
+        $this->$field = $newId;
+      }
+      
+      $result = 1;
     }
     
-    if ($newId !== null && ($field = $this->metadata->getSequenceColumn()) !== null) {
-      $this->$field = $newId;
+    if ($this->hasMethod("afterInsert")) {
+      $afterResult = $this->afterInsert($result, "save");
+      if ($afterResult !== null) $result = $afterResult;
     }
+    
+    return $result;
   }
   
   /**
    * @throws Sabel_DB_Exception
    * @return array
    */
-  protected function _saveUpdate()
+  protected function saveUpdate()
   {
-    if (($pkey = $this->metadata->getPrimaryKey()) === null) {
-      $message = "cannot update a model(there is not primary key).";
-      throw new Sabel_DB_Exception($message);
-    } else {
+    $result = null;
+    if ($this->hasMethod("beforeUpdate")) {
+      $result = $this->beforeUpdate("save");
+    }
+    
+    if ($result === null) {
+      if (($pkey = $this->metadata->getPrimaryKey()) === null) {
+        $message = __METHOD__ . "() can't update a model(there is not primary key).";
+        throw new Sabel_DB_Exception($message);
+      }
+      
       foreach ((is_string($pkey)) ? array($pkey) : $pkey as $key) {
-        $this->setCondition($this->modelName . "." . $key, $this->__get($key));
+        if ($this->primaryKeyValues[$key] === $this->$key) {
+          $this->setCondition("{$this->modelName}.{$key}", $this->__get($key));
+        } else {
+          $message = __METHOD__ . "() can't update the primary key value.";
+          throw new Sabel_DB_Exception($message);
+        }
+      }
+      
+      $saveValues = array();
+      foreach ($this->updateValues as $k => $v) {
+        $saveValues[$k] = (isset($columns[$k])) ? $columns[$k]->cast($v) : $v;
+      }
+      
+      $vColumn = ($this->versionColumn === "") ? "version" : $this->versionColumn;
+      
+      if (isset($this->metaCols[$vColumn])) {
+        $_column = $this->metaCols[$vColumn];
+        $currentVersion = $this->__get($vColumn);
+        $this->setCondition("{$this->modelName}.{$vColumn}", $currentVersion);
+        
+        if ($_column->isInt()) {
+          $saveValues[$vColumn] = $currentVersion + 1;
+        } elseif ($_column->isDatetime()) {
+          $saveValues[$vColumn] = now();
+        } else {
+          $message = __METHOD__ . "() version column must be DATETIME or INT.";
+          throw new Sabel_DB_Exception($message);
+        }
+      }
+      
+      $this->updateValues = array();
+      $stmt = $this->prepareStatement(Sabel_DB_Statement::UPDATE);
+      $result = $this->prepareUpdate($stmt, $saveValues)->execute();
+      
+      if (isset($this->metaCols[$vColumn]) && $result < 1) {
+        $message = __METHOD__ . "() this model has already been changed by other transactions.";
+        throw new Sabel_DB_Exception_StaleModel($message);
       }
     }
     
-    $saveValues = array();
-    foreach ($this->updateValues as $k => $v) {
-      $saveValues[$k] = (isset($columns[$k])) ? $columns[$k]->cast($v) : $v;
+    if ($this->hasMethod("afterUpdate")) {
+      $afterResult = $this->afterUpdate($result, "save");
+      if ($afterResult !== null) $result = $afterResult;
     }
     
-    $this->updateValues = array();
-    $stmt = $this->prepareStatement(Sabel_DB_Statement::UPDATE);
-    return $this->prepareUpdate($stmt, $saveValues)->execute();
+    return $result;
   }
   
   /**
@@ -769,37 +727,51 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   public function insert(array $values = array())
   {
-    return $this->prepare("insert", array($values))->execute();
-  }
-  
-  /**
-   * @return mixed
-   */
-  protected function _insert()
-  {
-    @list ($values) = $this->arguments;
-    $stmt = $this->prepareStatement(Sabel_DB_Statement::INSERT);
-    return $this->prepareInsert($stmt, $values)->execute();
+    $result = null;
+    if ($this->hasMethod("beforeInsert")) {
+      $result = $this->beforeInsert("insert");
+    }
+    
+    if ($result === null) {
+      $stmt   = $this->prepareStatement(Sabel_DB_Statement::INSERT);
+      $result = $this->prepareInsert($stmt, $values)->execute();
+    }
+    
+    if ($this->hasMethod("afterInsert")) {
+      $afterResult = $this->afterInsert($result, "insert");
+      if ($afterResult !== null) $result = $afterResult;
+    }
+    
+    if ($this->autoReinit) $this->initState();
+    
+    return $result;
   }
   
   /**
    * @param array $values
    *
-   * @return void
+   * @return int
    */
   public function update(array $values = array())
   {
-    $this->prepare("update", array($values))->execute();
-  }
-  
-  /**
-   * @return void
-   */
-  protected function _update()
-  {
-    @list ($values) = $this->arguments;
-    $stmt = $this->prepareStatement(Sabel_DB_Statement::UPDATE);
-    $this->prepareUpdate($stmt, $values)->execute();
+    $result = null;
+    if ($this->hasMethod("beforeUpdate")) {
+      $result = $this->beforeUpdate("update");
+    }
+    
+    if ($result === null) {
+      $stmt   = $this->prepareStatement(Sabel_DB_Statement::UPDATE);
+      $result = $this->prepareUpdate($stmt, $values)->execute();
+    }
+    
+    if ($this->hasMethod("afterUpdate")) {
+      $afterResult = $this->afterUpdate($result, "update");
+      if ($afterResult !== null) $result = $afterResult;
+    }
+    
+    if ($this->autoReinit) $this->initState();
+    
+    return $result;
   }
   
   /**
@@ -810,38 +782,43 @@ abstract class Sabel_DB_Model extends Sabel_Object
    */
   public function delete($arg1 = null, $arg2 = null)
   {
-    return $this->prepare("delete", array($arg1, $arg2))->execute();
-  }
-  
-  /**
-   * @throws Sabel_DB_Exception
-   * @return int
-   */
-  protected function _delete()
-  {
-    $condition = $this->getCondition();
-    @list ($arg1, $arg2) = $this->arguments;
-    
-    if (!$this->isSelected() && $arg1 === null && $condition->isEmpty()) {
-      $stmt = $this->prepareStatement(Sabel_DB_Statement::DELETE);
-      return $this->prepareDelete($stmt)->execute();
+    $result = null;
+    if ($this->hasMethod("beforeDelete")) {
+      $result = $this->beforeDelete("delete");
     }
     
-    if ($arg1 !== null) {
-      $this->setCondition($arg1, $arg2);
-    } elseif ($this->isSelected()) {
-      if (($pkey = $this->metadata->getPrimaryKey()) === null) {
-        $message = __METHOD__ . "() cannot delete model(there is not primary key).";
-        throw new Sabel_DB_Exception($message);
+    if ($result === null) {
+      $condition = $this->getCondition();
+      if (!$this->isSelected() && $arg1 === null && $condition->isEmpty()) {
+        $stmt   = $this->prepareStatement(Sabel_DB_Statement::DELETE);
+        $result = $this->prepareDelete($stmt)->execute();
       } else {
-        foreach ((is_string($pkey)) ? array($pkey) : $pkey as $key) {
-          $this->setCondition($this->modelName . "." . $key, $this->__get($key));
+        if ($arg1 !== null) {
+          $this->setCondition($arg1, $arg2);
+        } elseif ($this->isSelected()) {
+          if (($pkey = $this->metadata->getPrimaryKey()) === null) {
+            $message = __METHOD__ . "() cannot delete model(there is not primary key).";
+            throw new Sabel_DB_Exception($message);
+          } else {
+            foreach ((is_string($pkey)) ? array($pkey) : $pkey as $key) {
+              $this->setCondition($this->modelName . "." . $key, $this->__get($key));
+            }
+          }
         }
+        
+        $stmt   = $this->prepareStatement(Sabel_DB_Statement::DELETE);
+        $result = $this->prepareDelete($stmt)->execute();
       }
     }
     
-    $stmt = $this->prepareStatement(Sabel_DB_Statement::DELETE);
-    return $this->prepareDelete($stmt)->execute();
+    if ($this->hasMethod("afterDelete")) {
+      $afterResult = $this->afterDelete($result, "delete");
+      if ($afterResult !== null) $result = $afterResult;
+    }
+    
+    if ($this->autoReinit) $this->initState();
+    
+    return $result;
   }
   
   /**
@@ -900,4 +877,35 @@ abstract class Sabel_DB_Model extends Sabel_Object
   {
     return $stmt->where($this->getCondition()->build($stmt));
   }
+  
+  /**
+   * @param Sabel_DB_Model $model
+   *
+   * @return void
+   */
+  protected function _doSelectOne(Sabel_DB_Model $model)
+  {
+    $stmt = $this->prepareStatement(Sabel_DB_Statement::SELECT);
+    $rows = $this->prepareSelect($stmt)->execute();
+    if (isset($rows[0])) $model->setProperties($rows[0]);
+  }
+  
+  /**
+   * @param array $rows
+   *
+   * @return Sabel_DB_Model[]
+   */
+  protected function toModels(array $rows)
+  {
+    $results = array();
+    $source  = MODEL($this->modelName);
+    
+    foreach ($rows as $row) {
+      $model = clone $source;
+      $model->setProperties($row);
+      $results[] = $model;
+    }
+    
+    return $results;
+  }
 }
